"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = require("aws-sdk");
const regexMatcher_1 = require("../lib/regexMatcher");
const chai_1 = require("chai");
const jsonata = require("jsonata");
const cognito = require("./cognito");
exports.awsSdkStepRunners = ({ region, constructorArgs, }) => [
    {
        willRun: regexMatcher_1.regexMatcher(/^I execute "([^"]+)" of the AWS ([^ ]+) SDK( with)?$/),
        run: async ([method, api, withArgs], step, runner, flightRecorder) => {
            let argument;
            if (withArgs) {
                if (!step.interpolatedArgument) {
                    throw new Error('Must provide argument!');
                }
                try {
                    argument = JSON.parse(step.interpolatedArgument);
                }
                catch (_a) {
                    throw new Error(`Failed to parse argument: ${step.interpolatedArgument}`);
                }
            }
            let extraArgs = {};
            const cognitoEnabled = flightRecorder.flags[cognito.cognitoAuthentication];
            if (cognitoEnabled) {
                const { secretAccessKey, identityId, accessKeyId, sessionToken, } = flightRecorder.settings[cognito.cognitoAuthentication];
                extraArgs = {
                    credentials: {
                        secretAccessKey,
                        identityId,
                        accessKeyId,
                        sessionToken,
                    },
                };
                await runner.progress(`AWS-SDK.${api}.auth`, extraArgs.credentials.identityId);
            }
            const args = {
                region,
                ...(constructorArgs && constructorArgs[api]),
                ...extraArgs,
            };
            const a = new AWS[api](args);
            await runner.progress(`AWS-SDK.${api}`, `${method}(${argument !== undefined ? JSON.stringify(argument) : ''})`);
            const res = await a[method](argument).promise();
            runner.store.awsSdk = {
                res,
            };
            return res;
        },
    },
    {
        willRun: regexMatcher_1.regexMatcher(/^(?:"([^"]+)" of )?(?:the execution result|"([^"]+)") should (equal|match) this JSON$/),
        run: async ([exp, storeName, equalOrMatch], step, runner) => {
            const { awsSdk } = runner.store;
            if (!step.interpolatedArgument) {
                throw new Error('Must provide argument!');
            }
            const j = JSON.parse(step.interpolatedArgument);
            const result = storeName ? runner.store[storeName] : awsSdk.res;
            const fragment = exp ? jsonata(exp).evaluate(result) : result;
            if (equalOrMatch === 'match') {
                chai_1.expect(fragment).to.containSubset(j);
            }
            else {
                chai_1.expect(fragment).to.deep.equal(j);
            }
            return [fragment];
        },
    },
    {
        willRun: regexMatcher_1.regexMatcher(/^(?:"([^"]+)" of )?the execution result should equal ([0-9]+)$/),
        run: async ([exp, num], _, runner) => {
            const { awsSdk } = runner.store;
            const result = awsSdk.res;
            const fragment = exp ? jsonata(exp).evaluate(result) : result;
            chai_1.expect(fragment).to.equal(parseInt(num, 10));
            return [fragment];
        },
    },
    {
        willRun: regexMatcher_1.regexMatcher(/^I store "([^"]+)" of the execution result as "([^"]+)"$/),
        run: async ([expression, storeName], _, runner) => {
            const e = jsonata(expression);
            const { awsSdk } = runner.store;
            const result = e.evaluate(awsSdk.res);
            chai_1.expect(result).to.not.be.an('undefined');
            runner.store[storeName] = result;
            return result;
        },
    },
    {
        willRun: regexMatcher_1.regexMatcher(/^I parse "([^"]+)" of the execution result into "([^"]+)"$/),
        run: async ([expression, storeName], _, runner) => {
            const e = jsonata(expression);
            const { awsSdk } = runner.store;
            const result = e.evaluate(awsSdk.res);
            chai_1.expect(result).to.not.be.an('undefined');
            runner.store[storeName] = JSON.parse(result);
            return runner.store[storeName];
        },
    },
    {
        willRun: regexMatcher_1.regexMatcher(/^I escape this JSON into "([^"]+)"$/),
        run: async ([storeName], step, runner) => {
            if (!step.interpolatedArgument) {
                throw new Error('Must provide argument!');
            }
            const j = JSON.parse(step.interpolatedArgument);
            runner.store[storeName] = JSON.stringify(JSON.stringify(j));
            return runner.store[storeName];
        },
    },
];
